@@ -1,20 +1,31 @@
 /* certreq.c - create pkcs-10 messages
- *      Copyright (C) 2002, 2011 g10 Code GmbH
+ *      Copyright (C) 2002, 2011, 2012 g10 Code GmbH
  *
  * This file is part of KSBA.
  *
  * KSBA is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Fountion; either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of either
  *
- * KSBA is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *   - the GNU Lesser General Public License as published by the Free
+ *     Software Foundation; either version 3 of the License, or (at
+ *     your option) any later version.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * or
+ *
+ *   - the GNU General Public License as published by the Free
+ *     Software Foundation; either version 2 of the License, or (at
+ *     your option) any later version.
+ *
+ * or both in parallel, as here.
+ *
+ * KSBA is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
+ * License for more details.
+ *
+ * You should have received a copies of the GNU General Public License
+ * and the GNU Lesser General Public License along with this program;
+ * if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -161,7 +172,7 @@ ksba_certreq_set_issuer (ksba_certreq_t cr, const char *name)
 {
   if (!cr || !name)
     return gpg_error (GPG_ERR_INV_VALUE);
-  if (cr->subject.der)
+  if (cr->x509.issuer.der)
     return gpg_error (GPG_ERR_CONFLICT); /* Already set */
   return _ksba_dn_from_str (name, &cr->x509.issuer.der,
                             &cr->x509.issuer.derlen);
